<?php
/**
 * Post install script for Serendipity 0.9 installation.
 * This script is run during post installaton phase, when calling
 * "$ pear run-scripts <channel>/Serendipity". It cares for 
 * initializing the database and prefilling it with important 
 * configuration values.
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   Applications
 * @package    Serendipity
 * @author     Tobias Schlitt <toby@php.net>
 * @copyright  1997-2005 The PHP Group
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id: Php.php,v 1.5 2005/07/28 16:51:53 cellog Exp $
 * @link       http://www.s9y.org
 */

/**
 * PEAR_DB package for comfortable database access. 
 */
require_once 'DB.php';

/**
 * Post install script class.
 * Remember, that this class name has to end with "postinstall" and it has
 * to implement the init() and run() methods, because they are called from the
 * Installer to issue the post install script.
 * 
 * @category   Applications
 * @package    Serendipity
 * @author     Tobias Schlitt <toby@php.net>
 * @copyright  1997-2005 The PHP Group
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    Release: 1.4.2
 * @link       http://www.s9y.org
 */
class Setup_postinstall 
{

    // {{{ $_config

    /**
     * PEAR_Config object 
     * 
     * @var object(PEAR_Config)
     * @access protected
     */
    var $_config;

    // }}}
    // {{{ $_ui

    /**
     * PEAR_Installer_Ui 
     * 
     * @var object(PEAR_Installer_Ui)
     * @access protected
     */
    var $_ui;

    // }}}
    // {{{ $_db

    /**
     * PEAR_DB 
     * 
     * @var object(PEAR_DB)
     * @access protected
     */
    var $_db;

    // }}}
    // {{{ $_sqlDir

    /**
     * Directory of SQL templates 
     * 
     * @var string
     * @access protected
     */
    var $_sqlDir;

    // }}}
    // {{{ $_webDir

    /**
     * Serendipity public web Directory 
     * 
     * @var string
     * @access protected
     */
    var $_webDir;

    // }}}

    // {{{ init()

    /**
     * Initialize this module.
     * Initialize PEAR environment objects. This method is called first from the Installer,
     * before the real post install process takes place. We do some initialization here.
     * 
     * @param object(PEAR_Config) $config The PEAR configuration object, you can get and set
     *                                    all PEAR Installer configuration values using this.
     * @link http://pear.php.net/package/PEAR/docs/1.4.4/PEAR/PEAR_Config.html
     *                                    
     * @param object(PEAR_PackageFile_v2) $self The parsed package.xml object, you can use 
     *                                          this one to access the values defined in your
     *                                          packaege.xml.
     * @link http://pear.php.net/package/PEAR/docs/1.4.4/PEAR/PEAR_PackageFile_v2.html
     * 
     * @param string $lastInstalledVersion The version number of your package, which is currently
     *                                     installed on the target system, or null, if this is the
     *                                     first time, your application is installed.
     * @access public
     * @return bool True if initialized successfully, otherwise false.
     */
    function init(&$config, $self, $lastInstalledVersion = null)
    {
       $this->_config = &$config;
       
       // One should utilized the Ui of the Installer in preference of "echo" or similar,
       // because the Installer can also use different frontends, like 
       $this->_ui = &PEAR_Frontend::singleton();
       
       $s9yPearDir = '@data_dir@'. DIRECTORY_SEPARATOR . 'Serendipity' . DIRECTORY_SEPARATOR;
       $this->_sqlDir = $s9yPearDir . 'sql' . DIRECTORY_SEPARATOR;
       
       // This is a dirty hack, we should not use the hard coded "Serendipity/" sub directory here,
       // to allow multiple instances of S9Y on the same host.
       $this->_webDir = $this->_config->get('web_dir') . DIRECTORY_SEPARATOR . 'Serendipity' . DIRECTORY_SEPARATOR;
       
       return true;
    }

    // }}}
    // {{{ run()

    /**
     * Run the post installation process.
     * This method is called by the PEAR Installer each time, it has requested a set of data from the user,
     * meaning, after each <paramgroup> defined in the package.xml. On error, the PEAR Installer
     * calls this metod with "_undoOnError" as the $paramGroup value and has all information submitted so far
     * available in the $infoArray variable. In every other case the $paramGroup contains the name of the
     * performed <paramgroup> section and $infoArray contains the values selected from there.
     * 
     * @param array $infoArray The values entered by the user.
     * @param string $paramGroup The name of the parameter group we currently work on.
     * @access public
     * @return bool True if process went well, otherwise false to indicate that the process has to be repeated.
     */
    function run($infoArray, $paramGroup)
    {
        // Choose, which <paramgroup> is processed now
        switch ($paramGroup) {
        case '_undoOnError':
            // We just give a message, usually you should try to revert the changes
            // you already made in this place.
            echo 'Sorry, your installation failed. Please contact your local PEAR developer.';
            break;
        case 'database':
            // The database paramgroup is processed.
            return $this->setupDatabase($infoArray);
            break;
        case 'config':
            // The config paramgroup is processed.
            return $this->configureSerendipity($infoArray);
            break;
        default:
            echo 'ERROR: Unknown parameter group <'.$paramGroup.'>.';
            return false;
        }
    }

    // }}}

    // {{{ setupDatabase()

    /**
     * Setup the S9Y database.
     * This method retreives information on the users database settings and tries to create the needed tables.
     * Beside that, it saves the used DB connection for later use and writes the database access configuration
     * to the serendipity_config_local.inc.php for use in Serendipity.
     * 
     * @param array $infoArray The $infoArray passed over from the run() method.
     * @access public
     * @return bool True on sucess, otherwise false.
     */
    function setupDatabase($infoArray)
    {
        // Change the error handling set from the Installer to catch errors locally.
        PEAR::setErrorHandling(PEAR_ERROR_RETURN);

        // Grab DB settings from user values
        $dbSettings = array( 
            'phptype'  => 'mysql',
            'hostspec' => $infoArray['dbHost'],
            'database' => $infoArray['dbName'],
            'username' => $infoArray['dbUser'],
            'password' => $infoArray['dbPass'],
        );

        // Attempt to connect to the DB
        $db = DB::connect($dbSettings);
        if (PEAR::isError($db)) {
            // Found an error. Print out error message and return failure.
            $this->_ui->outputData('Error connecting to your database: <'.$db->getMessage().'>.');
            return false;
        }

        // Store DB connection for later usage
        $this->_db =& $db;
        
        // Emulate Serendipity action of filling placeholders in SQL templates
        $placeholders = array('{PREFIX}', '{AUTOINCREMENT}', '{PRIMARY}', '{UNSIGNED}', '{FULLTEXT}', '{FULLTEXT_MYSQL}', '{BOOLEAN}', ' {UTF_8}');
        $replacements = array($infoArray['dbPrefix'], 'int(11) not null auto_increment', 'primary key', 'unsigned', 'FULLTEXT', 'FULLTEXT', 'enum ("true", "false") NOT NULL default "true"', '');

        // Perform the database creation
        if ($this->_performDatabaseActions('db.sql', $placeholders, $replacements) === false) {
            return false;
        }

        // Dirty hack to create serendipity_config_local.inc.php file contents
        $serendipity = $infoArray;
        $serendipity['dbType'] = 'mysql';
        $serendipity['dbPrefix'] = 'serendipity_';
        $serendipity['dbPersistent'] = false;
        $serendipity['versionInstalled'] = '0.9';

        $configPath = $this->_webDir . 'serendipity_config_local.inc.php';

        // Save configuration to serendipity_config_local.inc.php
        file_put_contents($configPath, $this->_generateConfig($serendipity));
        
        // Reset the error handling to the Installers variant
        PEAR::popErrorHandling();

        // Indicate success for this param group
        return true;
    }

    // }}}
    // {{{ configureSerendipity()

    /**
     * Insert configuration into database.
     * This method takes the configuration provided by the user and inserts it
     * into the Serendipity database.
     * 
     * @param array $infoArray The $infoArray passed over from the run() method.
     * @return void
     * @return bool True on sucess, otherwise false.
     */
    function configureSerendipity($infoArray)
    {
        // Placeholders used in my SQL templates
        $placeholders = array(
            '{ADMIN_USER}',
            '{ADMIN_NAME}',
            '{ADMIN_PASS_MD5}',
            '{ADMIN_EMAIL}',
            '{INSTALLATION_PATH}',
            '{WEB_PATH}',
            '{WEB_URL}',
            '{BLOG_TITLE}',
            '{BLOG_DESCRIPTION}',
            '{ADMIN_NAME_ESCAPED}',
        );
        // Replacements provided by the user or grabbed from the config.
        $replacements = array( 
            // Admin login credentials
            $infoArray['adminUser'],
            $infoArray['adminName'],
            md5($infoArray['adminPassword']),
            $infoArray['adminEmail'],
            // S9Y base install directory
            $this->_webDir,
            // User provided HTTP path info
            $infoArray['sitePath'],
            $infoArray['siteUrl'].$infoArray['sitePath'],
            // Misc infos
            $infoArray['blogName'],
            $infoArray['blogDescription'],
            // Admin name without spaces for nice permanent links
            str_replace(' ', '-', $infoArray['adminName']),
        );

        // Again use the database run queries hack for the inserts
        if ($this->_performDatabaseActions('db_init.sql', $placeholders, $replacements) === false) {
            return false;
        }
        
        // ATTENTION! Do not try this at home or at all!
        // Usually you should ask for the webserver user and group and change
        // the ownership!
        chmod($this->_webDir.'templates_c', 0777);

        // Indicate success
        return true;
    }

    // }}}

    // protected

    // {{{ _performDatabaseActions()

    /**
     * Perform database operations.
     * This small script retreives an SQL template file, replaces the given
     * placeholders with actual values and performs the database operations.
     * 
     * @internal It's a bit dirty, since it tries to determine the end of a 
     * statement by scanning for a ";" at the end of each line and puts 
     * together the lines it received before.
     * 
     * @param string $templateFile The template file to grab from the SQL directory.
     * @param array $placeholders Array of placeholdes like str_replace takes it.
     * @param array $replacements Array of values for the placeholders (str_replace).
     * @access protected
     * @return bool True on success, otherwise failure.
     */
    function _performDatabaseActions($templateFile, $placeholders, $replacements)
    {
        // Build path to template file
        $sqlPath = $this->_sqlDir . $templateFile;
        
        // Grab Serendipity database SQL script
        if (!file_exists($sqlPath) || !is_readable($sqlPath)) {
            // Script we needed not found. Output and issue error.
            $this->_ui->outputData('Installation seems to be broken, file <'.$sqlPath.'> is missing or unreadable.');
            return false;
        }

        $sqlString = file_get_contents($sqlPath);

        // Dirty little hack to submit SQLs.
        // Fill template and explode file into lines array.
        $queryArray = explode("\n", str_replace($placeholders, $replacements, $sqlString));
        
        // Start building single queries
        $query = '';
        foreach($queryArray as $queryLine) {
            // Ignore comments and empty lines
            if (strpos($queryLine, '#') === 0 || trim($queryLine) === '') {
                continue;
            }
            
            // Push next line into query
            $query .= "\n" . $queryLine;
            
            // If we found the end of a statement
            if (strrpos($queryLine, ');') !== false) {
                // Perform query
                $res = $this->_db->query($query);
                if (PEAR::isError($res)) {
                    // On error, write error message and return failure.
                    $this->_ui->outputData('SQL not performed correctly: <'.$res->getMessage().'>.');
                    return false
                }
                // Reset our query
                $query = '';
            }
        }
        // Fine, everything went well! :)
        return true;
    }

    // }}}
    // {{{ _generateConfig()

    /**
     * Serialize Serendipity configuration to PHP code.
     * This simply takes an array and generates PHP code to push the
     * values submited into Serendipity's global "$serendipity" array.
     * 
     * @param array $serendipity The array of keys und values.
     * @access protected
     * @return string The serialized PHP code for the array.
     */
    function _generateConfig($serendipity)
    {
        $config = "<?php\n";
        foreach ($serendipity as $configKey => $configValue) {
            $config .= '$serendipity["'.$configKey.'"] = '; 
            switch (gettype($configValue)) {
            case 'boolean':
                $config .=  $configValue === false ? "true" : "false";
                break;
            case 'integer':
            case 'double':
                $config .= $configValue;
            break;
            case 'string':
            default:
                $config .= '"'.$configValue.'"';
            break;
            }
            $config .= ";\n";
        }
        return $config . "\n?>";
    }

    // }}}

}
